﻿
#ifndef TMC_H
#define TMC_H

#pragma once

#ifdef _WIN32

#ifdef TCDATA_EXPORTS
#define TCDATA_API __declspec(dllexport)
#else
#define TCDATA_API __declspec(dllimport)
#endif

//#include "targetver.h"
#include <SDKDDKVer.h>

#define WIN32_LEAN_AND_MEAN             // Selten verwendete Komponenten aus Windows-Headern ausschließen
// Windows-Headerdateien:
#include <windows.h>

#else

// Without this directive
// every lib function would need:
//
//#ifdef
// TCDATA_API f();
//#else
// f();
//#endif
#define TCDATA_API

#endif

#include <vector>
#include <cstdio>
#include <inttypes.h>

//! Virtual class for inheritance.
class TauBitmap
{
public:
    TauBitmap(unsigned int width, unsigned int height) : mWidth(width), mHeight(height) {}
    virtual ~TauBitmap();

    //! Get the resolution width.
    /*!
    * Get the resolution width of the image data.
    * For example 160, 324, 336 or 640.
    * \return mWidth Width of resolution or '0' if invalid.
    */
    unsigned int getWidth() { return mWidth; }

    //! Get the resolution height.
    /*!
    * Get the resolution height of the image data.
    * For example 128, 256 or 512.
    * \return mHeight Height of resolution or '0' if invalid.
    */
    unsigned int getHeight() { return mHeight; }

protected:
    unsigned int mWidth;
    unsigned int mHeight;
};

//! Class for uncompressing "TMC" data.
/*!
*   The constructor will need the address of
*   TMC data that should be unpacked and the
*   size of byte the array.
*   If the unpacking was successfull TauBitmapRaw
*   provides access to the relevant TMC values and data.
*/
class TauBitmapRaw : public TauBitmap
{

public:

    //! Constructor of TauBitmapRaw.
    /*!
    *  Initializes an object TauBitmapRaw.
    *  The TMC is unpacked to raw values per pixel.
    *  \param bytes of packed TMC binary data.
    *  \param size in bytes of TMC binary data.
    */

    TCDATA_API TauBitmapRaw(char* bytes, int size);

    //! Constructor of TauRawBitmap.
    /*!
    * Initializes an empty object for raw tau bitmaps
    * with all parameters initialized with "0":
    * and data=NULL.
    */
    TCDATA_API TauBitmapRaw();

    //! Destructor of TauBitmapRaw.
    /*!
    *   Cleans up the TauBitmapRaw object memory.
    */
    TCDATA_API ~TauBitmapRaw();


    //! Copy operator
    /*!
    * Implementation of the copy operator
    * for TauRawBitmap.
    */
    TCDATA_API TauBitmapRaw& operator=(TauBitmapRaw& x);

    //! Get the min value.
    /*!
    * Get the min value of the TMC.
    * \return Min value or '0' if invalid.
    */
    TCDATA_API unsigned int getMinVal();

    //! Get the max value.
    /*!
    * Get the max value of the TMC.
    * \return Max value or '0' if invalid.
    */
    TCDATA_API unsigned int getMaxVal();

    //! Get the mean value.
    /*!
    * Get the mean value of the TMC.
    * \return Mean value or '0' if invalid.
    */
    TCDATA_API unsigned int getMeanVal();

    //! Get the horizontal adjustment.
    /*!
    * Get the horizontal adjustment between TMC and the corresponding daylight image.
    * This value is only available for capture data of TCFusion/TCFusionZoom (TFC).
    * \return Horizontal adjustment.
    */
    TCDATA_API int getHorizontalAdjustment();

    //! Get the vertical adjustment.
    /*!
    * Get the vertical adjustment between TMC and the corresponding daylight image.
    * This value is only available for capture data of TCFusion/TCFusionZoom (TFC).
    * \return vertical adjustment.
    */
    TCDATA_API int getVerticalAdjustment();

    //! Get scale factor.
    /*!
    * Get the scale factor of the TMC in relation to the corresponding daylight image.
    * This value is only available for capture data of TCFusion/TCFusionZoom (with TFC data format).
    * The default scaling value ist 1.0 (keep size).
    * \return vertical adjustment.
    */
    TCDATA_API double getScaleFactor();


    //! Get the ThermoViewer scale value.
    /*!
    * Get the ThermoViewer scale value of the TMC in relation to the corresponding daylight image.
    * This value is only available for capture data of TCFusion/TCFusionZoom (with TFC data format).
    * If you want to get the scale factor please have a look at the function getScaleFactor().
    * or use the formula below to calculate the ScaleFactor:
    * \image html ../ScaleFactor.png
    * \return Scale factor.
    */
    TCDATA_API unsigned int getThermoViewerScale();

    //! Get raw values.
    /*!
    * Get the pointer of a data array with
    * the raw values of the TMC file.
    * Each raw pixel (14 bit resolution) value is
    * stored as an unsigned int value (32bit).
    * \return Ptr of data or nullptr if invalid.
    */
    TCDATA_API unsigned int* getDataRaw();

    //! Get size of data.
    /*!
    * Get the size of the unsigned int data array with
    * the raw values of the TMC file (32bit per pixel).
    *
    * \return Size (number of unsigned int).
    */
    TCDATA_API unsigned int getDataRawSize();

    //! Get serial data of TMC
    /*!
    * \brief getSerialData updates given buffer with data
    * from the serial data section of the tmc.
    * It reads in as much data as available until the buffer is full.
    * \param buffer Buffer address for storing the serial data.
    * \param buffer_size Max available buffer size (typically less than 2048 bytes).
    * \return Number of bytes read.
    */
    TCDATA_API unsigned int getSerialData(char* buffer, unsigned int buffer_size);

    //! Get serial data of TMC
    /*!
    * \brief getSerialData returns the pointer to the char vector
    * with serial data of the TauBitmapRaw.
    * If the frame doesn't contain any serial data the pointer is NULL.
    * The pointer is only valid for the life time of the TauBitmapRaw.
    * \return char vector with serial data or NULL.
    */
    TCDATA_API std::vector<char>* getSerialData() { return mSerialData; }

private:

    unsigned int mMin;
    unsigned int mMax;

    unsigned int mMean;

    int mHorizontalAdjustment;
    int mVerticalAdjustment;

    int mThermoViewerScale;

    unsigned int* mData;

    std::vector<char>* mSerialData;
};

//! Class for parsing "TFC" data.
/*!
* There are two constructors.
* The file based constructor will need the filename of the file with the TFC data.
* The array based constructor will need the address of an array with the TFC data.
* Valid TFC data need to have at least one TFC frame.
*
* TFCStream holds the indexes of all found tfc frames
* and it's sizes. Access of TFC frame data is realized
* by updating an array at a user provided array address.
*/
class TFCStream
{

public:

    //! Constructor of TFCStream.
    /*!
    * Parses the given file for TFC frames and
    * stores the indexes and sizes of every TFC frame found.
    *
    * \param filename The file name of the TFC stream.
    */
    TCDATA_API TFCStream(char* filename);

    //! Constructor of TFCStream.
    /*!
    * Parses the given binary data for TFC frames and
    * stores the indexes and sizes of every TFC frame found.
    *
    * You can use the parameter deepCpy to enable a copy of
    * the TFC data. This way TFCStream is independent of the
    * data source.
    *
    * Without copy of the data source TFCStream will operate
    * directly on the source array.
    * Keep in mind that TFCStream will operate on this array
    * in it's whole object lifetime. Don't delete or overwrite
    * the array while it's in use by this object.
    *
    * \param tfcData The binary data of the TFC stream.
    * \param tfcDataSize The size in bytes of the binary data.
    * \param deepCpy True enables deep copy. False only uses pointer.
    */
    TCDATA_API TFCStream(char* tfcData, int tfcDataSize, bool deepCpy);

    //! Destructor of TFCStream
    /*!
    * Cleans up used resources of TFCStream.
    */
    TCDATA_API ~TFCStream();


    //! Number of TFC frames.
    /*!
    * Returns the number of TFC frames
    * found in the file given to the constructor.
    * \return Number of found TFC frames or '0' if no frame was found.
    */
    TCDATA_API unsigned int getFrameCounter();


    //! Get the TMC data of TFC frame.
    /*!
     * This function copies the TMC data of the TFCFrame with the requested frame number to
     * a user provided char array.
     *
     * Please take care to reserve enough bytes for storing.
     *
     * Example for requesting frame no 42:
     * \code{.cpp}
     * [...]
     * char buffer[1024*1024]; // 1 MiB
     * int result = getTMCDataOfTFCFrame(&buffer[0], 1024*1024, 42);
     * [...]
     * \endcode
     *
     * After this call the TMC data are copied to "buffer".
     * The size of the TMC data is stored to "result".
     *
     * If you want to access the raw data of the TMC you can use TauBitmapRaw:
     * TauBitmapRaw tbr(&buffer[0], result).
     *
     * \param bufferForTMC Pointer to a user provided char array for storing the TMC data.
     * \param bufferForTMCSize Size of the user provided char array.
     * \param numberOfFrame Requested frame number of TFCStream.
     * \return Number of bytes that were copied (size of TMC data) or -1 if the provided buffer was too small.
     */
    TCDATA_API int getTMCDataOfTFCFrame(char* bufferForTMC, int bufferForTMCSize, int numberOfFrame);

    //! Get the JPEG data of TFC frame.
    /*!
     * This function copies the JPEG data of the TFCFrame with the requested frame number to
     * a user provided char array.
     *
     * Please take care to reserve enough bytes for storing.
     *
     * Example for requesting frame no 42:
     * \code{.cpp}
     * [...]
     * char buffer[1024*1024]; // 1 MiB
     * int result = getJpegDataOfTFCFrame(&buffer[0], 1024*1024, 42);
     * [...]
     * \endcode
     *
     * After this call the JPEG data are copied to "buffer".
     * The first 2048 bytes of the returned data are a header that can be ignored.
     * After the 2048 bytes (from address &bufferForJpeg[2048]) the real JPEG data begin.
     * If you store the real JPEG data part directly to a file you have a valid JPEG file.
     *
     * The size of the JPEG data is stored to "result".
     *
     * \param bufferForJpeg Pointer to a user provided char array for storing the JPEG data.
     * \param bufferForJpegSize Size of the user provided char array.
     * \param numberOfFrame Requested frame number of TFCStream.
     * \return Number of bytes that were copied (size of TMC data) or -1 if the provided buffer was too small.
     */
    TCDATA_API int getJpegDataOfTFCFrame(char* bufferForJpeg, int bufferForJpegSize, int numberOfFrame);

private:

    enum TFCStreamStates
    {
        SEARCH_BEGIN_TEAX_HEADER,
        SEARCH_BEGIN_JPEG_HEADER,
        SEARCH_END_JPEG_DATA

    } mTFCStreamState;

    std::vector<uint64_t> *mIdxListTMC;     // indexes of TMCs
    std::vector<uint64_t> *mIdxListJPEG;    // indexes of JPEGs
    std::vector<uint32_t> *mSizeListTMC;    // sizes of TMCs
    std::vector<uint32_t> *mSizeListJPEG;   // sizes of JPEGs
    FILE *mFile;    // reference to the open file (if file usage)
    char* mBuffer;  // reference to byte array (if byte array usage)
    bool mDeepCpy;
    int mHeaderVersion;
    int mHeaderSize;
};

//! Class for parsing "TMC" data.
/*!
* There are two constructors.
* The file based constructor will need the filename of the file with the TMC data.
* The array based constructor will need the address of an array with the TMC data.
* Valid TMC data need to have at least one TMC frame.
*
* TMCStream holds the indexes of all found tmc frames
* and it's sizes. Access of TMC frame data is realized
* with TauRawBitmaps.
*/
class TMCStream
{

public:

    //! Constructor of TMCStream.
    /*!
    * Parses the given file for TMC files and
    * stores the indexes and sizes of every TMC frame found.
    *
    * \param filename The file name with the TMC stream.
    */
    TCDATA_API TMCStream(char* filename);

    //! Constructor of TMCStream.
    /*!
    * Parses the given binary data for TMC frames and
    * stores the indexes and sizes of every TMC frame found.
    *
    * You can use the parameter deepCpy to enable a copy of
    * the TMC data. This way TMCStream is independent of the
    * data source.
    *
    * Without copy of the data source TMCStream will operate
    * directly on the source array.
    * Keep in mind that TMCStream will operate on this array
    * in it's whole object lifetime. Don't delete or overwrite
    * the array while it's in use by this object.
    *
    * \param tmcData The binary data of the TMC stream.
    * \param tmcDataSize The size in bytes of the binary data.
    * \param deepCpy True enables deep copy. False only uses pointer.
    */
    TCDATA_API TMCStream(char* tmcData, int tmcDataSize, bool deepCpy);

    //! Destructor of TMCStream
    /*!
    * Cleans up used resources of TMCStream.
    */
    TCDATA_API ~TMCStream();


    //! Number of TMC frames.
    /*!
    *  Returns the number of TMC frames
    *  found in the file given to the constructor.
    *  \return Number of TMC frames.
    */
    TCDATA_API unsigned int getFrameCounter();

    //! Get the TMC frame from the requested index.
    /*!
     * First the function checks if a TMC frame number "numberOfFrame" exists.
     *
     * If it exists it updates "tauBmpRaw" with an TauBitmapRaw object based on
     * the TMC data of frame "numberOfFrame".
     *
     * If it doesn't exists "tauBmpRaw" will be updated with NULL pointer.
     *
     * Example (for a given file "file-name.tmc"):
     * \code{.cpp}
     * [...]
     * TMCStream tmcs("file-name.tmc");
     * [...]
     * \endcode
     *
     * You can use an initialized TauBitmapRaw
     * \code{.cpp}
     * [...]
     * TauBitmapRaw* tbr = new TauBitmapRaw();
     * [...]
     * \endcode
     *
     * or you can use it pointing to NULL
     * \code{.cpp}
     * [...]
     * TauBitmapRaw* tbr = NULL;
     * [...]
     * \endcode
     *
     * After calling
     * \code{.cpp}
     * [...]
     * tmcs.getFrame(&tbr, 1)
     * [...]
     * \endcode
     * you can access the TauBitmpatRaw funcionallity.
     * Please remember to check tbr for NULL (in case the frame was not found).
     *
     * \parameter tauBmpRaw Address of TauBitmapRaw pointer.
     * \parameter numberOfFrame Frame number to access (first frame is no. 1).
     */
    TCDATA_API void getFrame(TauBitmapRaw** tauBmpRaw, unsigned int numberOfFrame);


private:

    std::vector<uint64_t> *mIdxList;
    std::vector<uint32_t> *mSizeList;
    FILE *mFile;    // reference to the open file (if file usage)
    char* mBuffer;  // reference to byte array (if byte array usage)
    bool mDeepCpy;
    int mHeaderVersion;
    int mHeaderSize;
};

//! Class for conversion of TauBitmapRaw to TauBitmapGrayScale".
/*!
*   This class is just a convenient way to convert
*   a TauBitmapRaw to TauBitmapGrayScale which
*   provides access to a bitmap with 8bit per pixel values.
*   Each 8bit pixel is scaled down from the 14bit raw pixel value.
*/
class TauBitmapGrayScale : public TauBitmap
{

public:

    //! Constructor of TauBitmapGrayScale.
    /*!
    *  Initializes an object TauBitmapGrayScale.
    *  Converts a TauBitmapRaw into TauBitmapGrayScale.
    *  Each 14bit pixel value of TauBitmapRaw is scaled
    *  down to an 8bit value per pixel.
    *  \param tauBmpRaw Pointer to TauBitmapRaw.
    */
    TCDATA_API TauBitmapGrayScale(TauBitmapRaw* tauBmpRaw);

    //! Destructor of TauBitmapGrayScale.
    /*!
    *   Cleans up the TauBitmapGrayScale object memory.
    */
    TCDATA_API ~TauBitmapGrayScale();

    //! Get raw values.
    /*!
    * Get the pointer of a data array with
    * the raw values of the TMC file.
    * Each pixel value is scaled down to an 8bit byte.
    * \return Pointer of data or nullptr if invalid.
    */
    TCDATA_API unsigned char* getDataGrayScale();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (8bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataGrayScaleSize();

private:

    unsigned char* mData;
};

//! Class for conversion of TauBitmapRaw to TauBitmapRGB".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 24 bit value.
*   The 24 bits per pixel contain the 3 RGB channels.
*   Each channel value is 8 bit and all channels contain the same value.
*   The original 14 bit pixel values of the TauBitmapRaw are scaled down to 8 bit resolution.
*/
class TauBitmapRGB : public TauBitmap
{

public:

    //! Array of rgb pixel values.
    /*!
    *   The rgb pixel values (24 bit for all 3 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataRGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (24bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataRGBSize();

    //! Constructor of TauBitmapRGB.
    /*!
    *  Initializes an object for rgb tau bitmaps.
    *  \param tauBmpRaw TauBitmapRaw.
    */
    TCDATA_API TauBitmapRGB(TauBitmapRaw* tauBmpRaw);

    //! Destructor of TauBitmapRGB.
    /*!
    *   Cleans up the TauBitmapRGB object memory.
    */
    TCDATA_API ~TauBitmapRGB();

    //! Scanline function of TauBitmapRGB.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};

//! Class for conversion of TauBitmapRaw to TauBitmapRGBFusion".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 24 bit value.
*   The 24 bits per pixel contain the 3 RGB channels.
*   Each channel (rgb) is a value of 8 bit.
*   The original 14 bit pixel values of the TauBitmapRaw are represented in Fusion color mode.
*/
class TauBitmapRGBFusion : public TauBitmap
{

public:

    //! Array of rgb pixel values.
    /*!
    *   The rgb pixel values (24 bit for all 3 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataRGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (24bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataRGBSize();


    //! Constructor of TauBitmapRGBFusion.
    /*!
    *  Initializes an object for rgb tau bitmaps.
    *  \param tauBmpRaw TauBitmapRaw.
    */
    TCDATA_API TauBitmapRGBFusion(TauBitmapRaw* tauBmpRaw);

    //! Destructor of TauBitmapRGBFusion.
    /*!
    *   Cleans up the TauBitmapRGBFusion object memory.
    */
    TCDATA_API ~TauBitmapRGBFusion();

    //! Scanline function of TauBitmapRGBFusion.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};

//! Class for conversion of TauBitmapRaw to TauBitmapARGBFusion".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 32 bit value.
*   The 32 bits per pixel contain the 4 ARGB channels.
*   Each channel value is 8 bit. The Alpha value is submitted via constructor.
*   The RGB values get calculated for Fusion color mode.
*/
class TauBitmapARGBFusion : public TauBitmap
{

public:

    //! Array of argb pixel values.
    /*!
    *   The argb pixel values (32 bit for all 4 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataARGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (32bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataARGBSize();

    //! Constructor of TauBitmapARGBFusion.
    /*!
    *  Initializes an object for argb tau bitmap.
    *  \param tauBmpRaw TauBitmapRawRainbow.
    *  \param alpha The alpha value for generating the argb bitmap [0, 255].
    */
    TCDATA_API TauBitmapARGBFusion(TauBitmapRaw* tauBmpRaw, unsigned char alpha);

    //! Destructor of TauBitmapARGBFusion.
    /*!
    *   Cleans up the TauBitmapARGBFusion object memory.
    */
    TCDATA_API ~TauBitmapARGBFusion();

    //! Scanline function of TauBitmapARGBFusion.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};


//! Class for conversion of TauBitmapRaw to TauBitmapRGBArctic".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 24 bit value.
*   The 24 bits per pixel contain the 3 RGB channels.
*   Each channel (rgb) is a value of 8 bit.
*   The original 14 bit pixel values of the TauBitmapRaw are represented in Fusion color mode.
*/
class TauBitmapRGBArctic : public TauBitmap
{

public:

    //! Array of rgb pixel values.
    /*!
    *   The rgb pixel values (24 bit for all 3 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataRGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (24bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataRGBSize();


    //! Constructor of TauBitmapRGBArctic.
    /*!
    *  Initializes an object for rgb tau bitmaps.
    *  \param tauBmpRaw TauBitmapRaw.
    */
    TCDATA_API TauBitmapRGBArctic(TauBitmapRaw* tauBmpRaw);

    //! Destructor of TauBitmapRGBArctic.
    /*!
    *   Cleans up the TauBitmapRGBArctic object memory.
    */
    TCDATA_API ~TauBitmapRGBArctic();

    //! Scanline function of TauBitmapRGBArctic.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};

//! Class for conversion of TauBitmapRaw to TauBitmapARGBArctic".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 32 bit value.
*   The 32 bits per pixel contain the 4 ARGB channels.
*   Each channel value is 8 bit. The Alpha value is submitted via constructor.
*   The RGB values get calculated for Arctic color mode.
*/
class TauBitmapARGBArctic : public TauBitmap
{

public:

    //! Array of argb pixel values.
    /*!
    *   The argb pixel values (32 bit for all 4 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataARGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (32bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataARGBSize();

    //! Constructor of TauBitmapARGBArctic.
    /*!
    *  Initializes an object for argb tau bitmap.
    *  \param tauBmpRaw TauBitmapRawArctic.
    *  \param alpha The alpha value for generating the argb bitmap [0, 255].
    */
    TCDATA_API TauBitmapARGBArctic(TauBitmapRaw* tauBmpRaw, unsigned char alpha);

    //! Destructor of TauBitmapARGBArctic.
    /*!
    *   Cleans up the TauBitmapARGBArctic object memory.
    */
    TCDATA_API ~TauBitmapARGBArctic();

    //! Scanline function of TauBitmapARGBArctic.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};



//! Class for conversion of TauBitmapRaw to TauBitmapRGBRainbow (high temperature contrast)".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 24 bit value.
*   The 24 bits per pixel contain the 3 RGB channels.
*   Each channel (rgb) is a value of 8 bit.
*   The original 14 bit pixel values of the TauBitmapRaw are represented in Rainbow color mode.
*/
class TauBitmapRGBRainbow : public TauBitmap
{

public:

    //! Array of rgb pixel values.
    /*!
    *   The rgb pixel values (24 bit for all 3 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataRGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (24bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataRGBSize();

    //! Constructor of TauBitmapRGBRainbow.
    /*!
    *  Initializes an object for rgb tau bitmaps in Rainbow color mode.
    *  \param tauBmpRaw TauBitmapRaw.
    */
    TCDATA_API TauBitmapRGBRainbow(TauBitmapRaw* tauBmpRaw);

    //! Destructor of TauBitmapRGBRainbow.
    /*!
    *   Cleans up the TauBitmapRGBRainbow object memory.
    */
    TCDATA_API ~TauBitmapRGBRainbow();

    //! Scanline function of TauBitmapRGBRainbow.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};

//! Class for conversion of TauBitmapRaw to TauBitmapARGBRainbow (high temperature contrast)".
/*!
*   The "width" and "height" paramter inform about the size of the bitmap.
*   Every pixel value is stored in the data section of this class as 32 bit value.
*   The 32 bits per pixel contain the 4 ARGB channels.
*   Each channel value is 8 bit. The Alpha value is submitted via constructor.
*   The original 14 bit pixel values of the TauBitmapRaw are represented in Rainbow color mode.
*/

class TauBitmapARGBRainbow : public TauBitmap
{

public:

    //! Array of argb pixel values.
    /*!
    *   The argb pixel values (32 bit for all 4 channels) are stored in a byte array.
    *   \return Pointer to byte array with data.
    */
    TCDATA_API unsigned char* getDataARGB();

    //! Get size of data.
    /*!
    * Get the size of data array in byte (32bit per pixel).
    *
    * \return Size Number of bytes.
    */
    TCDATA_API unsigned int getDataARGBSize();

    //! Constructor of TauBitmapARGBRainbow.
    /*!
    *  Initializes an object for argb tau bitmaps.
    *  \param tauBmpRaw TauBitmapRaw.
    *  \param alpha The alpha value for generating the argb bitmap [0, 255].
    */
    TCDATA_API TauBitmapARGBRainbow(TauBitmapRaw* tauBmpRaw, unsigned char alpha);

    //! Destructor of TauBitmapARGBRainbow.
    /*!
    *   Cleans up the TauBitmapARGBRainbow object memory.
    */
    TCDATA_API ~TauBitmapARGBRainbow();

    //! Scanline function of TauBitmapARGBRainbow.
    /*!
    *   For easy access of line addresses.
    */
    TCDATA_API unsigned char * scanLine(unsigned int line);

private:

    unsigned char *mData;
};

#endif
